(function ( $ ) {
    $.fn.flashOnce = function() {
        var target = this;
        this
            .addClass('flash-on')
            .delay(200) // this delay is linked to the transition in the flash-on CSS class.
            .queue(function() {
                target
                    .removeClass('flash-on')
                    .addClass('flash-off')
                    .dequeue()
                ;})
            .delay(1000)  // this delay is just to clean up the flash-X classes.
            .queue(function() {
                target
                    .removeClass('flash-on flash-off')
                    .dequeue()
                ;})
        ;
        return this;
    };
}(jQuery));

/**
 * Shows a task in the #task-details div.
 */
function task_open(task_id, project_url) {
    if (task_id === undefined || project_url === undefined) {
        if (console) console.log("task_open(", task_id, project_url, ") called.");
        return;
    }

    $('#task-list').find('a').removeClass('active');
    $('#task-' + task_id).addClass('active');

    var task_url = '/attract/' + project_url + '/tasks/' + task_id;
    console.log('task_url is ' + task_url);

    $.get(task_url, function(task_data) {
        $('#task-details').html(task_data);
    }).fail(function(xhr) {
        if (console) {
            console.log('Error fetching task', task_id, 'from', task_url);
            console.log('XHR:', xhr);
        }
        $('#task-details').html(xhr.responseText);
    });
}

/**
 * Create a task and show it in the #task-details div.
 */
function task_create(shot_id, project_url, task_type) {
    if (shot_id === undefined || project_url === undefined || task_type === undefined) {
        if (console) console.log("task_create(", shot_id, project_url, task_type, ") called.");
        return;
    }
    var url = '/attract/' + project_url + '/tasks/create';

    data = {
        task_type: task_type,
        parent: shot_id,
    };

    $.post(url, data, function(task_data) {
        task_open(task_data.task_id, project_url);
    })
    .fail(function(xhr) {
        if (console) {
            console.log('Error creating task');
            console.log('XHR:', xhr);
        }
        $('#task-details').html(xhr.responseText);
    });
}


function task_save(task_id, task_url) {
    console.log('Saving task to', task_url);

    var $form = $('#task-view form');
    var $button = $form.find("button[type='submit']");
    var payload = $form.serialize();
    var task = '#task-' + task_id;
    var $task = $(task);

    $button.attr('disabled', true);
    $task.addClass('processing');
    $('#status-bar').text('Saving task...');

    if (console) console.log('Sending:', payload);

    $.post(task_url, payload)
        .done(function(saved_task) {
            if (console) console.log('Done saving', saved_task);

            // Update the task list.
            // NOTE: this is tightly linked to the HTML of the task list in for_project.jade.
            $('.task-name-' + saved_task._id).text(saved_task.name).flashOnce();
            $task.find('span.name').text(saved_task.name);
            $task.find('span.type').text(saved_task.task_type);
            $task.find('span.status').text(saved_task.properties.status.replace('_', ' '));
            $task
                .removeClass('col-list-item task-list-item')
                .addClass('col-list-item task-list-item status-' + saved_task.properties.status);

            $('#status-bar').text('Saved task. ' + saved_task._updated);
        })
        .fail(function(xhr_or_response_data) {
            // jQuery sends the response data (if JSON), or an XHR object (if not JSON).
            if (console) console.log('Failed saving', xhr_or_response_data);
            $('#task-details').html(xhr_or_response_data.responseText);
            $('#status-bar').text('Failed saving. ' + xhr_or_response_data.responseText);
        })
        .always(function() {
            $button.attr('disabled', false);
            $task.removeClass('processing');
        })
    ;

    return false; // prevent synchronous POST to current page.
}
